import { ApolloClient, ApolloQueryResult } from "@apollo/client";
import { Link, MinilinksResult } from "./minilinks";
export declare const ALLOWED_IDS: number[];
export declare const DENIED_IDS: number[];
export declare const GLOBAL_ID_PACKAGE = 2;
export declare const GLOBAL_ID_CONTAIN = 3;
export declare const GLOBAL_ID_ANY = 8;
export declare const GLOBAL_ID_PROMISE = 9;
export declare const GLOBAL_ID_THEN = 10;
export declare const GLOBAL_ID_RESOLVED = 11;
export declare const GLOBAL_ID_REJECTED = 12;
export declare const GLOBAL_ID_PACKAGE_ACTIVE = 42;
export declare const GLOBAL_ID_PACKAGE_VERSION = 43;
export interface DeepClientOptions<L = Link<number>> {
    apolloClient: ApolloClient<any>;
    minilinks?: MinilinksResult<L>;
    table?: string;
    returning?: string;
    selectReturning?: string;
    insertReturning?: string;
    updateReturning?: string;
    deleteReturning?: string;
    defaultSelectName?: string;
    defaultInsertName?: string;
    defaultUpdateName?: string;
    defaultDeleteName?: string;
}
export interface DeepClientResult<R> extends ApolloQueryResult<R> {
}
export declare type DeepClientPackageSelector = string;
export declare type DeepClientPackageContain = string;
export declare type DeepClientLinkId = number;
export declare type DeepClientStartItem = DeepClientPackageSelector | DeepClientLinkId;
export declare type DeepClientPathItem = DeepClientPackageContain;
export interface DeepClientInstance<L = Link<number>> {
    apolloClient?: ApolloClient<any>;
    minilinks?: MinilinksResult<L>;
    table?: string;
    returning?: string;
    selectReturning?: string;
    insertReturning?: string;
    updateReturning?: string;
    deleteReturning?: string;
    defaultSelectName?: string;
    defaultInsertName?: string;
    defaultUpdateName?: string;
    defaultDeleteName?: string;
    stringify(any?: any): string;
    select<LL = L>(exp: any, options?: {
        table?: string;
        returning?: string;
        variables?: any;
        name?: string;
    }): Promise<DeepClientResult<LL[]>>;
    insert<LL = L>(objects: Partial<LL> | Partial<LL>[], options?: {
        table?: string;
        returning?: string;
        variables?: any;
        name?: string;
    }): Promise<DeepClientResult<{
        id: any;
    }[]>>;
    update(exp: any, value: any, options?: {
        table?: string;
        returning?: string;
        variables?: any;
        name?: string;
    }): Promise<DeepClientResult<{
        id: any;
    }[]>>;
    delete(exp: any, options?: {
        table?: string;
        returning?: string;
        variables?: any;
        name?: string;
    }): Promise<DeepClientResult<{
        returning: {
            id: any;
        }[];
    }>>;
    reserve<LL = L>(count: number): Promise<number[]>;
    await(id: number): Promise<boolean>;
    serializeWhere(exp: any, env?: string): any;
    serializeQuery(exp: any, env?: string): any;
    id(start: DeepClientStartItem, ...path: DeepClientPathItem[]): Promise<number>;
}
export declare class DeepClient<L = Link<number>> implements DeepClientInstance<L> {
    apolloClient?: ApolloClient<any>;
    minilinks?: MinilinksResult<L>;
    table?: string;
    returning?: string;
    selectReturning?: string;
    insertReturning?: string;
    updateReturning?: string;
    deleteReturning?: string;
    defaultSelectName?: string;
    defaultInsertName?: string;
    defaultUpdateName?: string;
    defaultDeleteName?: string;
    constructor(options: DeepClientOptions<L>);
    stringify(any?: any): string;
    select<LL = L>(exp: any, options?: {
        table?: string;
        returning?: string;
        variables?: any;
        name?: string;
    }): Promise<DeepClientResult<LL[]>>;
    insert<LL = L>(objects?: Partial<LL> | Partial<LL>[], options?: {
        table?: string;
        returning?: string;
        variables?: any;
        name?: string;
    }): Promise<DeepClientResult<{
        id: any;
    }[]>>;
    update(exp: any, value: any, options?: {
        table?: string;
        returning?: string;
        variables?: any;
        name?: string;
    }): Promise<DeepClientResult<{
        id: any;
    }[]>>;
    delete(exp: any, options?: {
        table?: string;
        returning?: string;
        variables?: any;
        name?: string;
    }): Promise<DeepClientResult<{
        returning: {
            id: any;
        }[];
    }>>;
    reserve<LL = L>(count: number): Promise<number[]>;
    await(id: number): Promise<boolean>;
    _serialize: {
        link: {
            value: string;
            relations: {
                string: string;
                number: string;
                object: string;
                to: string;
                from: string;
                in: string;
                out: string;
                type: string;
                typed: string;
            };
        };
        value: {
            relations: {
                link: string;
            };
        };
    };
    _boolExpFields: {
        _and: boolean;
        _not: boolean;
        _or: boolean;
    };
    serializeWhere(exp: any, env?: string): any;
    serializeQuery(exp: any, env?: string): any;
    pathToWhere(start: DeepClientStartItem, ...path: DeepClientPathItem[]): any;
    id(start: DeepClientStartItem, ...path: DeepClientPathItem[]): Promise<number>;
}
export declare function useDeep(apolloClientProps?: ApolloClient<any>): DeepClient<Link<number>>;
