"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.generateDown = exports.generateUp = void 0;
const sql_1 = require("@deep-foundation/hasura/sql");
const permission_1 = require("./permission");
const generateUp = (options) => () => __awaiter(void 0, void 0, void 0, function* () {
    const { schemaName, tableName, valueType, customColumnsSql = '', customAfterSql = '', linkRelation, linksTableName, api } = options;
    yield api.sql((0, sql_1.sql) `
    CREATE TABLE ${schemaName}."${tableName}" (id bigint PRIMARY KEY, link_id bigint, ${customColumnsSql ? customColumnsSql : `value ${valueType}`});
    CREATE SEQUENCE ${tableName}_id_seq
    AS bigint START WITH 1 INCREMENT BY 1 NO MINVALUE NO MAXVALUE CACHE 1;
    ALTER SEQUENCE ${tableName}_id_seq OWNED BY ${schemaName}."${tableName}".id;
    ALTER TABLE ONLY ${schemaName}."${tableName}" ALTER COLUMN id SET DEFAULT nextval('${tableName}_id_seq'::regclass);
    CREATE INDEX IF NOT EXISTS ${tableName}__id_hash ON ${tableName} USING hash (id);
    CREATE INDEX IF NOT EXISTS ${tableName}__link_id_hash ON ${tableName} USING hash (link_id);
    CREATE INDEX IF NOT EXISTS ${tableName}__link_id_btree ON ${tableName} USING btree (link_id);
    ${customColumnsSql ? '' : `CREATE INDEX IF NOT EXISTS ${tableName}__value_btree ON ${tableName} USING btree (value);`}
    ${customAfterSql}
  `);
    yield api.query({
        type: 'track_table',
        args: {
            schema: schemaName,
            name: tableName,
        },
    });
    yield api.query({
        type: 'create_object_relationship',
        args: {
            table: tableName,
            name: 'link',
            type: 'one_to_one',
            using: {
                manual_configuration: {
                    remote_table: {
                        schema: schemaName,
                        name: linksTableName,
                    },
                    column_mapping: {
                        link_id: 'id',
                    },
                    insertion_order: 'after_parent',
                },
            },
        },
    });
    yield api.query({
        type: 'create_object_relationship',
        args: {
            table: linksTableName,
            name: linkRelation,
            type: 'one_to_one',
            using: {
                manual_configuration: {
                    remote_table: {
                        schema: schemaName,
                        name: tableName,
                    },
                    column_mapping: {
                        id: 'link_id',
                    },
                    insertion_order: 'after_parent',
                },
            },
        },
    });
    yield (0, permission_1.permissions)(api, tableName, {
        select: {},
        insert: {},
        update: {},
        delete: {},
    });
});
exports.generateUp = generateUp;
const generateDown = (options) => () => __awaiter(void 0, void 0, void 0, function* () {
    const { schemaName, tableName, linkRelation, linksTableName, api } = options;
    yield api.query({
        type: 'drop_relationship',
        args: {
            table: linksTableName,
            relationship: linkRelation,
        },
    });
    yield api.query({
        type: 'drop_relationship',
        args: {
            table: tableName,
            relationship: 'link',
        },
    });
    yield api.query({
        type: 'untrack_table',
        args: {
            table: {
                schema: schemaName,
                name: tableName,
            },
        },
    });
    yield api.sql((0, sql_1.sql) `
    DROP TABLE ${schemaName}."${tableName}";
  `);
});
exports.generateDown = generateDown;
//# sourceMappingURL=type-table.js.map